package p;

import atomicsets.OrderedLock;

class AbstractList_SubList extends AbstractList {
    /*atomic(L)*/ private AbstractList l/*L=this.L*/;
    /*atomic(L)*/ private int offset;
    /*atomic(L)*/ private int size;
    /*atomic(L)*/ private int expectedModCount;

    public int getSize_internal(){
    	return size;
    }

	public int getSize() {
		synchronized (this.$lock_L) {
			return size;
		}
	}
    
    public void setSize_internal(int s){
    	size = s;
    }

	public void setSize(int s) {
		synchronized (this.$lock_L) {
			size = s;
		}
	}
    
    public int getOffSet_internal(){
    	return offset;
    }

	public int getOffSet() {
		synchronized (this.$lock_L) {
			return offset;
		}
	}
    
    public void setExpectedModCount_internal(int e){
    	expectedModCount = e;
    }

	public void setExpectedModCount(int e) {
		synchronized (this.$lock_L) {
			expectedModCount = e;
		}
	}
    
    public AbstractList getL_internal(){
    	return (AbstractList) l;
    }

	public AbstractList getL() {
		synchronized (this.$lock_L) {
			return (AbstractList) l;
		}
	}
    
    AbstractList_SubList(AbstractList list, int fromIndex, int toIndex) {
		this(list, fromIndex, toIndex, new OrderedLock());
	}

	AbstractList_SubList(AbstractList list/*L=this.L*/, int fromIndex, int toIndex, OrderedLock L) {
        super(L);
		if (fromIndex < 0)
            throw new IndexOutOfBoundsException("fromIndex = " + fromIndex);
        if (toIndex > list.size_internal())
            throw new IndexOutOfBoundsException("toIndex = " + toIndex);
        if (fromIndex > toIndex)
            throw new IllegalArgumentException("fromIndex(" + fromIndex +
                                               ") > toIndex(" + toIndex + ")");
        l = list;
        offset = fromIndex;
        size = toIndex - fromIndex;
        expectedModCount = l.getModCount_internal();
    }

    public Object set_internal(int index, Object element) {
        rangeCheck_internal(index);
        checkForComodification_internal();
        return l.set_internal(index+offset, element);
    }

	public Object set(int index, Object element) {
		synchronized (this.$lock_L) {
			rangeCheck_internal(index);
			checkForComodification_internal();
			return l.set_internal(index+offset, element);
		}
	}

    public Object get_internal(int index) {
        rangeCheck_internal(index);
        checkForComodification_internal();
        return l.get_internal(index+offset);
    }

	public Object get(int index) {
		synchronized (this.$lock_L) {
			rangeCheck_internal(index);
			checkForComodification_internal();
			return l.get_internal(index+offset);
		}
	}

    public int size_internal() {
        checkForComodification_internal();
        return size;
    }

	public int size() {
		synchronized (this.$lock_L) {
			checkForComodification_internal();
			return size;
		}
	}

    public void add_internal(int index, Object element) {
        if (index<0 || index>size)
            throw new IndexOutOfBoundsException();
        checkForComodification_internal();
        l.add_internal(index+offset, element);
        expectedModCount = l.getModCount_internal();
        size++;
        incrementModCount_internal();
    }

	public void add(int index, Object element) {
		synchronized (this.$lock_L) {
			if (index<0 || index>size)
			    throw new IndexOutOfBoundsException();
			checkForComodification_internal();
			l.add_internal(index+offset, element);
			expectedModCount = l.getModCount_internal();
			size++;
			incrementModCount_internal();
		}
	}

    public Object remove_internal(int index) {
        rangeCheck_internal(index);
        checkForComodification_internal();
        Object result = l.remove_internal(index+offset);
        expectedModCount = l.getModCount_internal();
        size--;
        incrementModCount_internal();
        return result;
    }

	public Object remove(int index) {
		synchronized (this.$lock_L) {
			rangeCheck_internal(index);
			checkForComodification_internal();
			Object result = l.remove_internal(index+offset);
			expectedModCount = l.getModCount_internal();
			size--;
			incrementModCount_internal();
			return result;
		}
	}

    protected void removeRange_internal(int fromIndex, int toIndex) {
        checkForComodification_internal();
        l.removeRange_internal(fromIndex+offset, toIndex+offset);
        expectedModCount = l.getModCount_internal();
        size -= (toIndex-fromIndex);
        incrementModCount_internal();
    }

	protected void removeRange(int fromIndex, int toIndex) {
		synchronized (this.$lock_L) {
			checkForComodification_internal();
			l.removeRange_internal(fromIndex+offset, toIndex+offset);
			expectedModCount = l.getModCount_internal();
			size -= (toIndex-fromIndex);
			incrementModCount_internal();
		}
	}

    public boolean addAll_internal(/*unitfor(L)*/ Collection c) {
        return addAll_internal(size, c);
    }

	public boolean addAll(Collection c) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = c.getLockForL();
			OrderedLock l4 = this.$lock_L;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				return addAll_internal(size, c);
			}
		}
	}

    public boolean addAll_internal(int index, /*unitfor(L)*/ Collection c) {
        if (index<0 || index>size)
            throw new IndexOutOfBoundsException(
                "Index: "+index+", Size: "+size);
        int cSize = c.size_internal();
        if (cSize==0)
            return false;

        checkForComodification_internal();
        l.addAll_internal(offset+index, c);
        expectedModCount = l.getModCount_internal();
        size += cSize;
        incrementModCount_internal();
        return true;
    }

	public boolean addAll(int index, Collection c) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = c.getLockForL();
			OrderedLock l4 = this.$lock_L;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				if (index<0 || index>size)
				    throw new IndexOutOfBoundsException(
				        "Index: "+index+", Size: "+size);
				int cSize = c.size_internal();
				if (cSize==0)
				    return false;
				checkForComodification_internal();
				l.addAll_internal(offset+index, c);
				expectedModCount = l.getModCount_internal();
				size += cSize;
				incrementModCount_internal();
			}
		}
		return true;
	}

    public Iterator iterator_internal() {
        return listIterator_internal();
    }

	public Iterator iterator() {
		synchronized (this.$lock_L) {
			return listIterator_internal();
		}
	}

    public ListIterator listIterator_internal(final int index) {
        checkForComodification_internal();
        if (index<0 || index>size)
            throw new IndexOutOfBoundsException(
                "Index: "+index+", Size: "+size);

        return (ListIterator) new AbstractList_SubList_1/*I=this.L*/(this, index, this.$lock_L);
    }

	public ListIterator listIterator(final int index) {
		synchronized (this.$lock_L) {
			checkForComodification_internal();
			if (index<0 || index>size)
			    throw new IndexOutOfBoundsException(
			        "Index: "+index+", Size: "+size);
			return (ListIterator) new AbstractList_SubList_1/*I=this.L*/(this, index, this.$lock_L);
		}
	}
    
    public List subList_internal(int fromIndex, int toIndex) { 
        return (List) new AbstractList_SubList/*L=this.L*/(this, fromIndex, toIndex, this.$lock_L);
    }

	public List subList(int fromIndex, int toIndex) {
		synchronized (this.$lock_L) {
			return (List) new AbstractList_SubList/*L=this.L*/(this, fromIndex, toIndex, this.$lock_L);
		}
	}

    private void rangeCheck_internal(int index) {
        if (index<0 || index>=size)
            throw new IndexOutOfBoundsException("Index: "+index+
                                                ",Size: "+size);
    }

	private void rangeCheck(int index) {
		synchronized (this.$lock_L) {
			if (index<0 || index>=size)
			    throw new IndexOutOfBoundsException("Index: "+index+
			                                        ",Size: "+size);
		}
	}

    private void checkForComodification_internal() {
        if (l.getModCount_internal() != expectedModCount)
            throw new ConcurrentModificationException();
    }

	private void checkForComodification() {
		synchronized (this.$lock_L) {
			if (l.getModCount_internal() != expectedModCount)
			    throw new ConcurrentModificationException();
		}
	}
}
