package p;

import atomicsets.OrderedLock;

/**
 * LinkedHashMap entry.
 */
class LinkedHashMap_Entry extends HashMap_Entry {
    // These fields comprise the doubly linked list used for iteration.
    /*atomic(F)*/ private LinkedHashMap_Entry before/*F=this.F*/;
    /*atomic(F)*/ private LinkedHashMap_Entry after/*F=this.F*/;

LinkedHashMap_Entry(int hash, Object key, Object value, HashMap_Entry next) {
		this(hash, key, value, next, new OrderedLock());
	}

LinkedHashMap_Entry(int hash, Object key, Object value, HashMap_Entry next/*F=this.F*/, OrderedLock F) {
        super(hash, key, value, next, F);
    }

	public LinkedHashMap_Entry/*F=this.F*/ getBefore_internal(){
		return before;
	}

	public LinkedHashMap_Entry getBefore() {
		synchronized (this.$lock_F) {
			return before;
		}
	}
	
	public void setBefore_internal(LinkedHashMap_Entry/*F=this.F*/ e){
		before = e;
	}

	public void setBefore(LinkedHashMap_Entry e) {
		synchronized (this.$lock_F) {
			before = e;
		}
	}
	 
	public LinkedHashMap_Entry/*F=this.F*/ getAfter_internal(){
		return after;
	}

	public LinkedHashMap_Entry getAfter() {
		synchronized (this.$lock_F) {
			return after;
		}
	}
	
	public void setAfter_internal(LinkedHashMap_Entry/*F=this.F*/ e){
		after = e;
	}

	public void setAfter(LinkedHashMap_Entry e) {
		synchronized (this.$lock_F) {
			after = e;
		}
	}

    /**
     * Remove this entry from the linked list.
     */
    private void remove_internal() {
        before.setAfter_internal(after);
        after.setBefore_internal(before);
    }

	/**
	 * Remove this entry from the linked list.
	 */
	private void remove() {
		synchronized (this.$lock_F) {
			before.setAfter_internal(after);
			after.setBefore_internal(before);
		}
	}

    /**                                             
     * Insert this entry before the specified existing entry in the list.
     */
    void addBefore_internal(LinkedHashMap_Entry existingEntry/*F=this.F*/) {
        after  = existingEntry;
        before = existingEntry.getBefore_internal();
        before.setAfter_internal(this);
        after.setBefore_internal(this);
    }

	/**
	 * Insert this entry before the specified existing entry in the list.
	 */
	void addBefore(LinkedHashMap_Entry existingEntry) {
		synchronized (this.$lock_F) {
			after  = existingEntry;
			before = existingEntry.getBefore_internal();
			before.setAfter_internal(this);
			after.setBefore_internal(this);
		}
	}

    /**
     * This method is invoked by the superclass whenever the value
     * of a pre-existing entry is read by Map.get or modified by Map.set.
     * If the enclosing Map is access-ordered, it moves the entry
     * to the end of the list; otherwise, it does nothing. 
     */
    void recordAccess_internal(HashMap m/*M=this.F*/) {
        LinkedHashMap lm/*M=this.F*/ = (LinkedHashMap/*M=this.F*/)m; // FT: add annotation?
        if (lm.getAccessOrder_internal()) {
            lm.setModCount_internal(lm.getModCount_internal());
            remove_internal();
            addBefore_internal(lm.getHeader_internal());
        }
    }

	/**
	 * This method is invoked by the superclass whenever the value of a pre-existing entry is read by Map.get or modified by Map.set. If the enclosing Map is access-ordered, it moves the entry to the end of the list; otherwise, it does nothing. 
	 */
	void recordAccess(HashMap m) {
		synchronized (this.$lock_F) {
			LinkedHashMap lm/*M=this.F*/ = (LinkedHashMap/*M=this.F*/)m; // FT: add annotation?
			if (lm.getAccessOrder_internal()) {
			    lm.setModCount_internal(lm.getModCount_internal());
			    remove_internal();
			    addBefore_internal(lm.getHeader_internal());
			}
		}
	}

    void recordRemoval_internal(HashMap m/*M=this.F*/) {
        remove_internal();
    }

	void recordRemoval(HashMap m) {
		synchronized (this.$lock_F) {
			remove_internal();
		}
	}
}