package p;

import atomicsets.OrderedLock;

/**
 * Node in the Tree.  Doubles as a means to pass key-value pairs back to
 * user (see Map.Entry).
 */

class TreeMap_Entry implements Map_Entry, atomicsets.Atomic {
	public final OrderedLock getLockForF() {
		return $lock_F;
	}

	public final OrderedLock getLock() {
		return this.getLockForF();
	}

	/*atomic(F)*/ private Object key;
	/*atomic(F)*/ private Object value;
	/*atomic(F)*/ private TreeMap_Entry left/*F=this.F*/ = null;
	/*atomic(F)*/ private TreeMap_Entry right/*F=this.F*/ = null;
	/*atomic(F)*/ private TreeMap_Entry parent/*F=this.F*/;
	/*atomic(F)*/ private boolean color = TreeMap.BLACK;

	public TreeMap_Entry/*F=this.F*/ getLeft_internal(){ // FT: added
		return left;
	}

	public TreeMap_Entry getLeft() {
		synchronized (this.$lock_F) {
			return left;
		}
	}
	
	public void setLeft_internal(TreeMap_Entry/*F=this.F*/ l){ // FT: added
		left = l;
	}

	public void setLeft(TreeMap_Entry l) {
		synchronized (this.$lock_F) {
			left = l;
		}
	}
	
	public TreeMap_Entry/*F=this.F*/ getRight_internal(){ // FT: added
		return right;
	}

	public TreeMap_Entry getRight() {
		synchronized (this.$lock_F) {
			return right;
		}
	}
	
	public void setRight_internal(TreeMap_Entry/*F=this.F*/ r){ // FT: added
		right = r;
	}

	public void setRight(TreeMap_Entry r) {
		synchronized (this.$lock_F) {
			right = r;
		}
	}
	
	public TreeMap_Entry/*F=this.F*/ getParent_internal(){ // FT: added
		return parent;
	}

	public TreeMap_Entry getParent() {
		synchronized (this.$lock_F) {
			return parent;
		}
	}
	
	public void setParent_internal(TreeMap_Entry/*F=this.F*/ p){ // FT: added
		parent = p;
	}

	public void setParent(TreeMap_Entry p) {
		synchronized (this.$lock_F) {
			parent = p;
		}
	}
	
	public boolean getColor_internal(){ // FT: added
		return color;
	}

	public boolean getColor() {
		synchronized (this.$lock_F) {
			return color;
		}
	}
	
	public void setColor_internal(boolean c){ // FT: added
		color = c;
	}

	public void setColor(boolean c) {
		synchronized (this.$lock_F) {
			color = c;
		}
	}
	
	
    /**
	 * Make a new cell with given key, value, and parent, and with <tt>null</tt> child links, and BLACK color.
	 */
	TreeMap_Entry(Object key, Object value, TreeMap_Entry parent) {
		this(key, value, parent, new OrderedLock());
	}

	/**
     * Make a new cell with given key, value, and parent, and with
     * <tt>null</tt> child links, and BLACK color.
     */
    TreeMap_Entry(Object key, Object value, TreeMap_Entry parent/*F=this.F*/, OrderedLock F) {
        $lock_F = F;
		this.key = key;
        this.value = value;
        this.parent = parent;
    }

    /**
     * Returns the key.
     *
     * @return the key.
     */
    public Object getKey_internal() {
        return key;
    }

	/**
	 * Returns the key.
	 * @return  the key.
	 */
	public Object getKey() {
		synchronized (this.$lock_F) {
			return key;
		}
	}
    
    public void setKey_internal(Object k){ // FT: added
    	key = k;
    }

	public void setKey(Object k) {
		synchronized (this.$lock_F) {
			key = k;
		}
	}

    /**
     * Returns the value associated with the key.
     *
     * @return the value associated with the key.
     */
    public Object getValue_internal() {
        return value;
    }

	/**
	 * Returns the value associated with the key.
	 * @return  the value associated with the key.
	 */
	public Object getValue() {
		synchronized (this.$lock_F) {
			return value;
		}
	}

    /**
     * Replaces the value currently associated with the key with the given
     * value.
     *
     * @return the value associated with the key before this method was
     *           called.
     */
    public Object setValue_internal(Object value) {
        Object oldValue = this.value;
        this.value = value;
        return oldValue;
    }

	/**
	 * Replaces the value currently associated with the key with the given value.
	 * @return  the value associated with the key before this method was called.
	 */
	public Object setValue(Object value) {
		synchronized (this.$lock_F) {
			Object oldValue = this.value;
			this.value = value;
			return oldValue;
		}
	}

    public boolean equals_internal(/*unitfor*/ Object o) {
        if (!(o instanceof Map_Entry))
            return false;
        Map_Entry e = (Map_Entry)o;

        return TreeMap.valEquals(key,e.getKey()) && TreeMap.valEquals(value,e.getValue());
    }

	public boolean equals(Object o) {
		if (!(o instanceof Map_Entry))
            return false;
		Map_Entry e = (Map_Entry)o;
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = ((atomicsets.Atomic) o).getLock();
			OrderedLock l4 = this.$lock_F;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				return TreeMap.valEquals(key,e.getKey()) && TreeMap.valEquals(value,e.getValue());
			}
		}
	}

    public int hashCode_internal() {
        int keyHash = (key==null ? 0 : key.hashCode());
        int valueHash = (value==null ? 0 : value.hashCode());
        return keyHash ^ valueHash;
    }

	public int hashCode() {
		synchronized (this.$lock_F) {
			int keyHash = (key==null ? 0 : key.hashCode());
			int valueHash = (value==null ? 0 : value.hashCode());
			return keyHash ^ valueHash;
		}
	}

    public String toString_internal() {
        return key + "=" + value;
    }

	public String toString() {
		synchronized (this.$lock_F) {
			return key + "=" + value;
		}
	}

	protected final OrderedLock $lock_F;
}