package de.laures.cewolf.util;

import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGeneratorContext;
import org.apache.batik.svggen.SVGGraphics2D;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

public class RendererHelper {

	/*atomicset(R)*/
	
	/*atomic(R)*/ private Object dummy = null;
	
	/**
	 * Handles rendering a chart as a PNG. Currently this method is synchronized
	 * because of concurrency issues with JFreeChart.
	 *
	 * @param  baos
	 * @param  chart
	 * @param  width
	 * @param  height
	 * @param  info
	 * @throws IOException
	 */
	/*synchronized*/ void handlePNG (ByteArrayOutputStream baos, JFreeChart chart,
			int width, int height, ChartRenderingInfo info)
		throws IOException {
			ChartUtilities.writeChartAsPNG(baos, chart, width, height, info);
	}

	/**
	 * Handles rendering a chart as a JPEG. Currently this method is synchronized
	 * because of concurrency issues with JFreeChart.
	 *
	 * @param  baos
	 * @param  chart
	 * @param  width
	 * @param  height
	 * @param  info
	 * @throws IOException
	 */
	/*synchronized*/ void handleJPEG (ByteArrayOutputStream baos, JFreeChart chart,
			int width, int height, ChartRenderingInfo info)
		throws IOException {
			ChartUtilities.writeChartAsJPEG(baos, chart, width, height, info);
	}

	/**
		 * Handles rendering a chart as a SVG. Currently this method is synchronized
		 * because of concurrency issues with JFreeChart.
		 *
		 * @param  baos
		 * @param  chart
		 * @param  width
		 * @param  height
		 * @throws IOException
		 */
		/*synchronized*/ void handleSVG (ByteArrayOutputStream baos, JFreeChart chart, int width, int height)
			throws IOException {
			OutputStreamWriter writer = new OutputStreamWriter(baos, "UTF-8");
			DOMImplementation domImpl = GenericDOMImplementation.getDOMImplementation();
			Document document = domImpl.createDocument("cewolf-svg", "svg", null);
			SVGGeneratorContext ctx = SVGGeneratorContext.createDefault(document);
	//		ctx.setComment("Generated by Cewolf using JFreeChart and Apache Batik SVG Generator");
			SVGGraphics2D svgGenerator = new SVGGraphics2D(ctx, false);
			svgGenerator.setSVGCanvasSize(new Dimension(width, height));
			chart.draw(svgGenerator, new Rectangle2D.Double(0, 0, width, height), null);
			svgGenerator.stream(writer, false);
			writer.close();
		}

}
